package jp.chiheisen;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.chiheisen.util.MapCounter;

/**
 * \bh𕪐͂Av쐬NXłB
 * 
 * @version 1.00, 10/10/10
 */
public class MethodNameAnalyzer {
	/** ړop^[ */
	private static final Pattern PREFIX_EXTRACTION = Pattern.compile("^([^A-Z]+)");
	/** Pꒊop^[ */
	private static final Pattern WORD_EXTRACTION = Pattern.compile("(.[^A-Z]+)");
	
	/** O\bh */
	private static final Set<String> IGNORE = new HashSet<String>(Arrays.asList(new String[]{
		"clone", "equals", "finalize",
		"getClass", "hashCode", "notify",
		"notifyAll", "toString", "wait",
		"fillInStackTrace", "getCause", "getLocalizedMessage",
		"getMessage", "getStackTrace", "initCause",
		"printStackTrace", "setStackTrace"}
	));

	/** ړ */
	private final Map<String, Integer> prefix;
	/** P */
	private final Map<String, Integer> word;
	/** ŒZ\bh */
	private final Set<String> min;
	/** Œ\bh */
	private final Set<String> max;
	
	/**
	 * \bh̓v쐬܂B
	 * 
	 * @param methodNameList \bh̃Xg
	 */
	public MethodNameAnalyzer(List<String> methodNameList){
		MapCounter<String> prefix = new MapCounter<String>();
		MapCounter<String> word = new MapCounter<String>();
		
		int minLen = Integer.MAX_VALUE;
		Set<String> min = new HashSet<String>();
		int maxLen = 0;
		Set<String> max = new HashSet<String>();
		
		for(String methodName : methodNameList){
			if(IGNORE.contains(methodName)){
				continue;
			}
			
			Matcher prefixMatcher = PREFIX_EXTRACTION.matcher(methodName);
			if(prefixMatcher.lookingAt()){
				prefix.increment(prefixMatcher.group(1));
			}
			
			Matcher wordMatcher = WORD_EXTRACTION.matcher(methodName);
			while(wordMatcher.find()){
				word.increment(wordMatcher.group(1).toLowerCase());
			}
			
			if(minLen >= methodName.length()){
				if(minLen > methodName.length()){
					minLen = methodName.length();
					min.clear();
				}
				min.add(methodName);
			}
			if(maxLen <= methodName.length()){
				if(maxLen < methodName.length()){
					maxLen = methodName.length();
					max.clear();
				}
				max.add(methodName);
			}
		}
		
		this.prefix = prefix.getResult();
		this.word = word.getResult();
		this.min = min;
		this.max = max;
	}
	
	/**
	 * \bh̐ړƓo񐔂̃}bv擾܂B
	 * 
	 * @return ړƓo񐔂̃}bv(ύXs)
	 */
	public Map<String, Integer> getPrefix() {
		return prefix;
	}

	/**
	 * \bhɊ܂܂ĂPƓo񐔂̃}bv擾܂B
	 * 
	 * @return PƓo񐔂̃}bv(ύXs)
	 */
	public Map<String, Integer> getWord() {
		return word;
	}
	
	/**
	 * ŒZ\bh擾܂B
	 * 
	 * @return ŒZ\bh
	 */
	public Set<String> getMin() {
		return min;
	}
	
	/**
	 * Œ\bh擾܂B
	 * 
	 * @return Œ\bh
	 */
	public Set<String> getMax() {
		return max;
	}
}
